def ninja_rule(ruleDict):
    """Return a list of Ninja rules

    Args:
        ruleDict (dict): a dictionary of rules
    Returns:
        a list, each item is a string like
        rule <rule name>
            command = ...
    """
    output = []
    for k in sorted(ruleDict.keys()):
        v = ruleDict[k]
        output.append(
            "rule {}\n  command = {}".format(k, v)
        )
    return "\n\n".join(output)



"""
Merge two dictionaries into one.
"""
from typing import Dict
import copy


def merge2(dict1, dict2):
    # type: (Dict, Dict) -> Dict
    """Recrusively merge two dictionaries.

    The two input dictionaries can have arbitrary depths.
    If there is a conflict, values from dict2 will
    override dict1.

    Args:
        dict1 (dict): input dictionary 1
        dict2 (dict): input dictionary 2

    Returns:
        A new dictionary that combines both inputs.

    """
    ooo = copy.deepcopy(dict1)
    for k in dict2:
        if (k in dict1 and  # merge dictionaries
                isinstance(dict1[k], dict) and
                isinstance(dict2[k], dict)):
            ooo[k] = merge2(dict1[k], dict2[k])
        elif (k in dict1 and  # merge lists
                isinstance(dict1[k], list) and
                isinstance(dict2[k], list)):
            ooo[k] = dict1[k] + dict2[k]
        else:  # dict2 has priority (add new or override)
            ooo[k] = dict2[k]
    return ooo



def ninja_io(args):
    """Return the inputs/outputs as a single string

    Args:
        args (str | list): a string or list of strings
    Returns:
        a string
    """
    if isinstance(args, str):
        return args
    elif isinstance(args, list):
        return " ".join(args)
    else:
        return ""



from ..fnTK import reduce
from .merge2 import merge2


def merge(dicts):
    """Merge many dictionaries into one
    Args:
        dicts (list): a list of dictionaries
    Returns:
        a dict
    """
    if len(dicts) == 0:
        return {}
    elif len(dicts) == 1:
        return dicts[0]
    else:
        return reduce(merge2, dicts, {})



from .ninja_const import ninja_const
from .ninja_rule import ninja_rule
from .ninja_build import ninja_build


def ninja_generate(argd):
    """Return the transformed Ninja build file content

    Args:
        argd (dict): user configuration dictionary
    Returns:
        a string (content of the Ninja build file)
    """
    return "\n\n\n".join(
            [
                ninja_const(argd['const']),
                ninja_rule(argd['rules']),
                ninja_build(argd['tasks'])
            ]
        )



from codecs import open as codecs_open
from setuptools import setup, find_packages


# Get the long description from the relevant file
with codecs_open('README.md', encoding='utf-8') as f:
    long_description = f.read()


setup(name='ninjag',
      version='0.1.3',
      description=u"Ninja config file generator",
      long_description=long_description,
      keywords='',
      author=u"Steven(Yuhang) Wang",
      url='https://github.com/yuhangwang/ninjag-python',
      license='MIT',
      packages=find_packages(exclude=['test']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'PyYAML',
          'typing'
      ],
      extras_require={
          'test': ['pytest'],
      },
      entry_points={
          'console_scripts': [
                  'ninjag = ninjag.cli:cli'
              ],
      }
      )



from .ninja_io import ninja_io


def ninja_const(inputDict, indent=""):
    """Return a list of user defined constants

    If the value of the const definition is a list,
    then it will be concatenated.

    Args:
        inputDict (dict): a dictionary of constant definitions
        indent (str): indentation for each definition

    Returns:
        a string
    """
    output = []
    for k in sorted(inputDict.keys()):
        v = inputDict[k]
        output.append(
                "{}{} = {}".format(indent, k, ninja_io(v))
            )
    return "\n".join(output)
